// Library:       MQSeries
// Component:     C++ sample Programs
// Part:          IMQSPUT.CPP
//
// Description:   Sample C++ program that puts messages to a named
//                queue.
//
//                IMQSPUT has 2 parameters:
//                - the name of a queue (required)
//                - the name of a queue manager (optional)
//
//                If no queue manager name is given, the queue manager
//                defaults to the default local queue manager.
//
// Statement:     Licensed Materials - Property of IBM
//                5621-390, 5765-623
//                (C) Copyright IBM Corp. 1994, 1996
//
//                U.S. Government Users Restricted Rights - use,
//                duplication or disclosure restricted by GSA ADP
//                Schedule Contract with IBM Corp.

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
}

#include <imqi.hpp> // MQSeries MQI

int main ( int argc, char * * argv ) {
  FILE * fp ;                      /* Console                       */

  ImqQueueManager mgr;             /* Queue manager                 */
  ImqQueue queue;                  /* Queue                         */
  ImqMessage msg;                  /* Data message                  */
  MQLONG   buflen;                 /* buffer length                 */
  char     buffer[100];            /* message buffer                */

  printf( "Sample IMQSPUT start\n" );
  if ( argc < 2 ) {
    printf( "Required parameter missing - queue name\n" );
    exit( (int)99 );
  }

  /******************************************************************/
  /*                                                                */
  /*   Connect to queue manager                                     */
  /*                                                                */
  /******************************************************************/
  if ( argc > 2 ) {
    mgr.setName( argv[2] );
  }

  if ( ! mgr.connect( ) ) {

    /* stop if it failed */
    printf( "ImqQueueManager::connect ended with reason code %ld\n",
            mgr.reasonCode( ) );
    exit( (int)mgr.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Associate queue with queue manager.                          */
  /*                                                                */
  /******************************************************************/
  queue.setConnectionReference( mgr );

  /******************************************************************/
  /*                                                                */
  /*   Use parameter as the name of the target queue                */
  /*                                                                */
  /******************************************************************/
  queue.setName( argv[ 1 ] );
  printf( "target queue is %s\n", argv[ 1 ] );

  /******************************************************************/
  /*                                                                */
  /*   Open the target message queue for output                     */
  /*                                                                */
  /******************************************************************/
  queue.setOpenOptions( MQOO_OUTPUT /* open queue for output        */
#if defined( _IMQ_VER ) && ( _IMQ_VER >= 200 )
    + MQOO_FAIL_IF_QUIESCING
#endif
                             );     /* but not if MQM stopping      */
  queue.open( );

  /* report reason, if any; stop if failed      */
  if ( queue.reasonCode( ) ) {
    printf( "ImqQueue::open ended with reason code %ld\n",
            queue.reasonCode( ) );
  }

  if ( queue.completionCode( ) == MQCC_FAILED ) {
    printf( "unable to open queue for output\n" );
  }

  /******************************************************************/
  /*                                                                */
  /*   Read lines from the file and put them to the message queue   */
  /*   Loop until null line or end of file, or there is a failure   */
  /*                                                                */
  /******************************************************************/
  fp = stdin;
  msg.useEmptyBuffer( buffer, sizeof( buffer ) );
  msg.setFormat( MQFMT_STRING );      /* character string format    */

  while ( queue.completionCode( ) != MQCC_FAILED ) {

    if ( fgets( buffer, sizeof(buffer) - 1, fp ) ) {
      buflen = strlen( buffer ) - 1 ; /* length without end-line    */
      buffer[ buflen ] = '\0' ;       /* remove end-line            */
    } else {
      buflen = 0;           /* treat EOF same as null line          */
    }

    /****************************************************************/
    /*                                                              */
    /*   Put each buffer to the message queue                       */
    /*                                                              */
    /****************************************************************/
    if ( buflen > 0 ) {
      msg.setMessageLength( buflen );
      if ( ! queue.put( msg ) ) {

        /* report reason, if any */
        printf( "ImqQueue::put ended with reason code %ld\n",
                queue.reasonCode( ) );
      }

    } else {
      /* quit loop when empty line is read */
      break ;
    }
  }

  /******************************************************************/
  /*                                                                */
  /*   Close the target queue (if it was opened)                    */
  /*                                                                */
  /******************************************************************/
  if ( ! queue.close( ) ) {

    /* report reason, if any     */
    printf( "ImqQueue::close ended with reason code %ld\n",
            queue.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Disconnect from MQM if not already connected (the            */
  /*   ImqQueueManager object handles this situation automatically) */
  /*                                                                */
  /******************************************************************/
  if ( ! mgr.disconnect( ) ) {

    /* report reason, if any     */
    printf( "ImqQueueManager::disconnect ended with reason code %ld\n",
            mgr.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /* END OF IMQSPUT                                                 */
  /*                                                                */
  /******************************************************************/
  printf( "Sample IMQSPUT end\n" );
  return( 0 );
}
