// Library:       MQSeries
// Component:     C++ sample Programs
// Part:          IMQSTRG.CPP
//
// Description:   Sample C++ trigger monitor program
//
//                IMQSTRG has 2 parameters:
//                - the name of an initiation queue (required)
//                - the name of a queue manager (optional)
//
//                If no queue manager name is given, the queue manager
//                defaults to the default local queue manager.
//
// Statement:     Licensed Materials - Property of IBM
//                5621-390, 5765-623
//                (C) Copyright IBM Corp. 1994, 1996
//
//                U.S. Government Users Restricted Rights - use,
//                duplication or disclosure restricted by GSA ADP
//                Schedule Contract with IBM Corp.
//
// Function:
//
//   IMQSTRG is a sample C++ program that acts as a trigger
//   monitor - reads an initiation queue, then starts
//   the program associated with each trigger message
//
//   Note - the function shown in this sample is a subset
//          of the full triggering function that is in
//          the supplied runmqtrm command.  Trigger messages
//          in error are not written to the Dead Letter Queue;
//          triggering only applies to the DEFault ApplType
//
//      -- reads messages from an initiation queue named in
//         the parameter to the trigger monitor
//
//      -- performs command for each valid trigger message
//
//         -- command runs command named in ApplId
//         -- parm = string version of MQTMC2 structure
//         -- EnvData is used by the trigger monitor as an
//            extension to the invoking command string
//
//      -- writes a message for each MQI reason other than
//         MQRC_NONE; stops if there is a MQI completion code
//         of MQCC_FAILED
//
// Program logic:
//      Use program parameter as the initiation queue name
//      MQOPEN queue for INPUT
//      while no MQI failures,
//      .  MQGET next message, remove from queue
//      .  invoke command based on trigger messages
//      .  .  ApplId   is name of program to call
//      .  .  MQTMC2   is parameter
//      MQCLOSE the trigger queue

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
}

#include <imqi.hpp> // MQSeries MQI

int main ( int argc, char * * argv ) {

  ImqQueueManager manager ;                    // Queue manager.
  ImqQueue queueInitiation ;                   // Initiation queue.
  ImqMessage msg ;                             // Message.
  ImqGetMessageOptions gmoInitiation ;         // Initiation queue get
                                               // message options.
  ImqTrigger trigger ;                         // Trigger message.

  ImqString strCommand ;           // Call command string ...
#if defined( _IMQ_VER ) && ( _IMQ_VER >= 200 )
  struct {                         // Trigger insert.
    MQTMC2 tmc2 ;                  // Text-only structure.
    char cNull ;                   // Trailing null.
  } sz2 ;
#endif

  MQLONG   mytype;                 /* saved application type        */

  printf( "Sample IMQSTRG start\n" );
  if ( argc < 2 ) {
    printf( "Required parameter missing - initiation queue name\n" );
    exit( (int)99 );               /* stop if no parameter          */
  }

  /******************************************************************/
  /*                                                                */
  /*   Associate the subject queue with the queue manager           */
  /*   Identify the subject queue                                   */
  /*                                                                */
  /******************************************************************/
  queueInitiation.setConnectionReference( manager );
  queueInitiation.setName( argv[ 1 ] );

  /******************************************************************/
  /*                                                                */
  /*   Connect to queue manager                                     */
  /*                                                                */
  /******************************************************************/
  if ( argc > 2 ) {
    manager.setName( argv[ 2 ] );
  }

  /* stop if it failed     */
  if ( ! manager.connect( ) ) {
    printf( "ImqQueueManager::connect failed with reason code %ld\n",
            manager.reasonCode( ) );
    exit( (int)manager.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*  Open specified initiation queue for input; exclusive or       */
  /*  shared use of the queue is controlled by the queue definition */
  /*                                                                */
  /******************************************************************/
  queueInitiation.setOpenOptions(
#if defined( _IMQ_VER ) && ( _IMQ_VER >= 200 )
      MQOO_INPUT_AS_Q_DEF         /* open queue for input           */
    + MQOO_FAIL_IF_QUIESCING      /* but not if MQM stopping        */
#else
      MQOO_INPUT_SHARED           /* open queue for input           */
#endif
    );
  queueInitiation.open( );
                                  /* report reason, if any          */
  if ( queueInitiation.reasonCode( ) ) {
    printf( "ImqQueue::open (%s) ==> %ld\n", (char *)queueInitiation.name( ),
            queueInitiation.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Get messages from the message queue                          */
  /*   Loop until there is a failure, ask to fail if the queue      */
  /*   manager is quiescing                                         */
  /*                                                                */
  /******************************************************************/
  gmoInitiation.setOptions(
      MQGMO_WAIT                       /* wait for new messages ... */
#if defined( _IMQ_VER ) && ( _IMQ_VER >= 200 )
    + MQGMO_FAIL_IF_QUIESCING          /* or until MQM stopping     */
#endif
    + MQGMO_ACCEPT_TRUNCATED_MSG );    /* remove all long messages  */
  gmoInitiation.setWaitInterval(
      MQWI_UNLIMITED );                /* no time limit             */

  while ( queueInitiation.completionCode( ) != MQCC_FAILED ) {

    /****************************************************************/
    /*                                                              */
    /*   In order to read the messages in sequence, MsgId and       */
    /*   CorrelID must have the default value.  MQGET sets them     */
    /*   to the values in for message it returns, so re-initialise  */
    /*   them before every call                                     */
    /*                                                              */
    /****************************************************************/
    msg.setMessageId( );
    msg.setCorrelationId( );

    /****************************************************************/
    /*                                                              */
    /*   Wait for a trigger                                         */
    /*   Process each message received                              */
    /*                                                              */
    /****************************************************************/
    if ( queueInitiation.get( msg, gmoInitiation ) &&
         trigger.pasteIn( msg ) ) {

      /* report reason, if any     */
      if ( queueInitiation.reasonCode( ) ) {
        printf( "ImqQueue::get ==> %ld\n", queueInitiation.reasonCode( ) );
      }

      /**************************************************************/
      /*                                                            */
      /*   Copy appropriate parts of trigger message into the       */
      /*   call command string                                      */
      /*                                                            */
      /**************************************************************/

      mytype = trigger.applicationType( );         /* save the type */

#if defined( _IMQ_VER ) && ( _IMQ_VER >= 200 )
      trigger.copyOut( & sz2.tmc2 );      /* builds MQTMC2, with    */
                                          /* blank queue manager    */
                                          /* name                   */
      ImqString::copy( sz2.tmc2.QMgrName,          /* fill in the   */
                       sizeof( sz2.tmc2.QMgrName ),/* queue manager */
                       (char *)manager.name( ),    /* name, padded  */
                       ' ' );                      /* with blanks.  */
      sz2.cNull = '\0' ;                  /* null after structure   */
#endif

      strCommand = trigger.applicationId( ) + (char *)" \"" +
#if defined( _IMQ_VER ) && ( _IMQ_VER >= 200 )
                   (char *) & sz2 +
#else
                   trigger.queueName( ) +
#endif
                   (char *)"\" " + trigger.environmentData( );

      /**************************************************************/
      /*                                                            */
      /*   Call the program                                         */
      /*                                                            */
      /**************************************************************/
      printf( "%s;\n", (char *)strCommand );
#if defined( __OS2__ )
      system( (char *)strCommand );
#endif

    }     /* end process for successful GET */
  }       /* end message processing loop    */

  /******************************************************************/
  /*                                                                */
  /*   Close the initiation queue - if it was opened                */
  /*                                                                */
  /******************************************************************/
  queueInitiation.close( );

  /* report reason, if any     */
  if ( queueInitiation.reasonCode( ) ) {
    printf( "ImqQueue::close ==> %ld\n", queueInitiation.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /*   Disconnect from MQM (unless previously connected)            */
  /*                                                                */
  /******************************************************************/
  manager.disconnect( );

  /* report reason, if any     */
  if ( manager.reasonCode( ) ) {
    printf( "ImqQueueManager::disconnect ended with reason code %ld\n",
            manager.reasonCode( ) );
  }

  /******************************************************************/
  /*                                                                */
  /* END OF IMQSTRG                                                 */
  /*                                                                */
  /******************************************************************/
  printf( "Sample IMQSTRG end\n" );
  return( (int)0 );
}
